@@ -5,8 +5,6 @@ Data Format (ASDF) files
 
 __all__ = [
     "AsdfFile",
-    "CustomType",
-    "AsdfExtension",
     "Stream",
     "open",
     "IntegerType",
@@ -19,15 +17,11 @@ __all__ = [
 ]
 
 
-from jsonschema import ValidationError
-
 from ._convenience import info
+from ._version import version as __version__
 from .asdf import AsdfFile
 from .asdf import open_asdf as open
 from .config import config_context, get_config
-from .extension import AsdfExtension
-from .stream import Stream
-from .tags.core import IntegerType
+from .exceptions import ValidationError
+from .tags.core import IntegerType, Stream
 from .tags.core.external_reference import ExternalArrayReference
-from .types import CustomType
-from .version import version as __version__
