@@ -5,7 +5,8 @@ Implementation of command for displaying available tags in asdf
 
 import sys
 
-from .. import AsdfFile
+from asdf import AsdfFile
+
 from .main import Command
 
 __all__ = ["list_tags"]
@@ -17,7 +18,9 @@ class TagLister(Command):  # pragma: no cover
     @classmethod
     def setup_arguments(cls, subparsers):
         parser = subparsers.add_parser(
-            str("tags"), help="List currently available tags", description="""Lists currently available tags."""
+            "tags",
+            help="List currently available tags",
+            description="""Lists currently available tags.""",
         )
 
         parser.add_argument(
@@ -39,8 +42,8 @@ class TagLister(Command):  # pragma: no cover
 def _format_type(typ):
     if isinstance(typ, str):
         return typ
-    else:
-        return "{}.{}".format(typ.__module__, typ.__name__)
+
+    return f"{typ.__module__}.{typ.__name__}"
 
 
 def list_tags(display_classes=False, iostream=sys.stdout):
@@ -50,8 +53,6 @@ def list_tags(display_classes=False, iostream=sys.stdout):
     tag_pairs = []
     for tag in af.extension_manager._converters_by_tag:
         tag_pairs.append((tag, af.extension_manager.get_converter_for_tag(tag).types))
-    for tag in af.type_index._type_by_tag:
-        tag_pairs.append((tag, [af.type_index._type_by_tag[tag]]))
 
     for tag, types in sorted(tag_pairs, key=lambda pair: pair[0]):
         string = str(tag)
