@@ -8,6 +8,7 @@ instead, one should use the factory function `get_file`.
 """
 
 import io
+import mmap
 import os
 import pathlib
 import re
@@ -26,6 +27,10 @@ from .util import patched_urllib_parse
 __all__ = ["get_file", "get_uri", "resolve_uri", "relative_uri"]
 
 
+_FILE_PERMISSIONS_DEFAULT_UMASK = 0o22
+_FILE_PERMISSIONS_ALL = 0o777
+_FILE_PERMISSIONS_NO_EXECUTE = 0o666
+
 _local_file_schemes = ["", "file"]
 if sys.platform.startswith("win"):  # pragma: no cover
     import string
@@ -67,7 +72,8 @@ def resolve_uri(base, uri):
     resolved = patched_urllib_parse.urljoin(base, uri)
     parsed = patched_urllib_parse.urlparse(resolved)
     if parsed.path != "" and not parsed.path.startswith("/"):
-        raise ValueError("Resolved to relative URL")
+        msg = "Resolved to relative URL"
+        raise ValueError(msg)
     return resolved
 
 
@@ -88,14 +94,11 @@ def relative_uri(source, target):
         return target
 
     if relative is None:
-        if tu[2] == su[2]:
-            relative = ""
-        else:
-            relative = os.path.relpath(tu[2], os.path.dirname(su[2]))
+        relative = "" if tu[2] == su[2] else os.path.relpath(tu[2], os.path.dirname(su[2]))
     if relative == ".":
         relative = ""
-    relative = patched_urllib_parse.urlunparse(["", "", relative] + extra)
-    return relative
+
+    return patched_urllib_parse.urlunparse(["", "", relative, *extra])
 
 
 class _TruncatedReader:
@@ -106,7 +109,14 @@ class _TruncatedReader:
     """
 
     def __init__(
-        self, fd, delimiter, readahead_bytes, delimiter_name=None, include=False, initial_content=b"", exception=True
+        self,
+        fd,
+        delimiter,
+        readahead_bytes,
+        delimiter_name=None,
+        include=False,
+        initial_content=b"",
+        exception=True,
     ):
         self._fd = fd
         self._delimiter = delimiter
@@ -132,33 +142,37 @@ class _TruncatedReader:
 
         if nbytes is None:
             content = self._fd.peek()
-        elif nbytes <= len(self._initial_content):
+        elif nbytes > len(self._initial_content):
+            content = self._fd.peek(nbytes - len(self._initial_content) + self._readahead_bytes)
+        else:
             content = self._initial_content[:nbytes]
             self._initial_content = self._initial_content[nbytes:]
+
             return content
-        else:
-            content = self._fd.peek(nbytes - len(self._initial_content) + self._readahead_bytes)
 
         if content == b"":
             if self._exception:
-                raise DelimiterNotFoundError("{0} not found".format(self._delimiter_name))
+                msg = f"{self._delimiter_name} not found"
+                raise DelimiterNotFoundError(msg)
+
             self._past_end = True
+
             return content
 
         index = re.search(self._delimiter, content)
         if index is not None:
-            if self._include:
-                index = index.end()
-            else:
-                index = index.start()
+            index = index.end() if self._include else index.start()
+
             content = content[:index]
             self._past_end = True
-        elif nbytes is None and self._exception:
-            # Read the whole file and didn't find the delimiter
-            raise DelimiterNotFoundError("{0} not found".format(self._delimiter_name))
-        else:
+
+        elif not (nbytes is None and self._exception):
             if nbytes:
                 content = content[: nbytes - len(self._initial_content)]
+        else:
+            # Read the whole file and didn't find the delimiter
+            msg = f"{self._delimiter_name} not found"
+            raise DelimiterNotFoundError(msg)
 
         self._fd.fast_forward(len(content))
 
@@ -208,7 +222,8 @@ class GenericFile(metaclass=util.InheritDocstrings):
             sources.
         """
         if not _check_bytes(fd, mode):
-            raise ValueError("File-like object must be opened in binary mode.")
+            msg = "File-like object must be opened in binary mode."
+            raise ValueError(msg)
 
         # can't import at the top level due to circular import
         from .config import get_config
@@ -218,7 +233,6 @@ class GenericFile(metaclass=util.InheritDocstrings):
         self._fd = fd
         self._mode = mode
         self._close = close
-        self._size = None
         self._uri = uri
 
         self.block_size = get_config().io_block_size
@@ -226,10 +240,10 @@ class GenericFile(metaclass=util.InheritDocstrings):
     def __enter__(self):
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, type_, value, traceback):
         if self._close:
             if hasattr(self._fd, "__exit__"):
-                self._fd.__exit__(type, value, traceback)
+                self._fd.__exit__(type_, value, traceback)
             else:
                 self._fd.close()
 
@@ -246,7 +260,8 @@ class GenericFile(metaclass=util.InheritDocstrings):
                 block_size = io.DEFAULT_BUFFER_SIZE
 
         if block_size <= 0:
-            raise ValueError(f"block_size ({block_size}) must be > 0")
+            msg = f"block_size ({block_size}) must be > 0"
+            raise ValueError(msg)
 
         self._blksize = block_size
 
@@ -320,7 +335,8 @@ class GenericFile(metaclass=util.InheritDocstrings):
             Must be 1D contiguous.
         """
         if len(array.shape) != 1 or not array.flags.contiguous:
-            raise ValueError("Requires 1D contiguous array.")
+            msg = "Requires 1D contiguous array."
+            raise ValueError(msg)
 
         self.write(array.data)
 
@@ -340,9 +356,11 @@ class GenericFile(metaclass=util.InheritDocstrings):
             cursor = self.tell()
             content = self.read(size)
             self.seek(cursor, SEEK_SET)
+
             return content
-        else:
-            raise RuntimeError("Non-seekable file")
+
+        msg = "Non-seekable file"
+        raise RuntimeError(msg)
 
     def seek(self, offset, whence=0):
         """
@@ -359,10 +377,9 @@ class GenericFile(metaclass=util.InheritDocstrings):
             SEEK_SET or 0 (absolute file positioning); other values
             are SEEK_CUR or 1 (seek relative to the current
             position) and SEEK_END or 2 (seek relative to the
-            file’s end).
+            file`s end).
         """
         result = self._fd.seek(offset, whence)
-        self.tell()
         return result
 
     def tell(self):
@@ -385,12 +402,13 @@ class GenericFile(metaclass=util.InheritDocstrings):
         """
         if self._close:
             self._fd.close()
+            self._fix_permissions()
 
     def truncate(self, size=None):
         """
         Truncate the file to the given size.
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def writable(self):
         """
@@ -424,7 +442,13 @@ class GenericFile(metaclass=util.InheritDocstrings):
         return self._fd.closed
 
     def read_until(
-        self, delimiter, readahead_bytes, delimiter_name=None, include=True, initial_content=b"", exception=True
+        self,
+        delimiter,
+        readahead_bytes,
+        delimiter_name=None,
+        include=True,
+        initial_content=b"",
+        exception=True,
     ):
         """
         Reads until a match for a given regular expression is found.
@@ -483,7 +507,13 @@ class GenericFile(metaclass=util.InheritDocstrings):
         return buff.getvalue()
 
     def reader_until(
-        self, delimiter, readahead_bytes, delimiter_name=None, include=True, initial_content=b"", exception=True
+        self,
+        delimiter,
+        readahead_bytes,
+        delimiter_name=None,
+        include=True,
+        initial_content=b"",
+        exception=True,
     ):
         """
         Returns a readable file-like object that treats the given
@@ -519,10 +549,16 @@ class GenericFile(metaclass=util.InheritDocstrings):
         DelimiterNotFoundError :
             If the delimiter is not found before the end of the file.
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def seek_until(
-        self, delimiter, readahead_bytes, delimiter_name=None, include=True, initial_content=b"", exception=True
+        self,
+        delimiter,
+        readahead_bytes,
+        delimiter_name=None,
+        include=True,
+        initial_content=b"",
+        exception=True,
     ):
         """
         Seeks in the file until a match for a given regular expression
@@ -576,18 +612,19 @@ class GenericFile(metaclass=util.InheritDocstrings):
         try:
             while reader.read(self.block_size) != b"":
                 pass
-            return True
-        except DelimiterNotFoundError as e:
+        except DelimiterNotFoundError:
             if exception:
-                raise e
-            else:
-                return False
+                raise
+
+            return False
+
+        return True
 
     def fast_forward(self, size):
         """
         Move the file position forward by `size`.
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def clear(self, nbytes):
         """
@@ -600,7 +637,7 @@ class GenericFile(metaclass=util.InheritDocstrings):
 
     def memmap_array(self, offset, size):
         """
-        Memmap a chunk of the file into a `np.core.memmap` object.
+        Memmap a chunk of the file into a `np.memmap` object.
 
         Parameters
         ----------
@@ -612,9 +649,25 @@ class GenericFile(metaclass=util.InheritDocstrings):
 
         Returns
         -------
-        array : np.core.memmap
+        array : np.memmap
         """
-        raise NotImplementedError()
+        msg = f"memmapping is not implemented for {self.__class__.__name__}"
+        raise NotImplementedError(msg)
+
+    def close_memmap(self):
+        """
+        Close the memmapped file (if one was mapped with memmap_array)
+        """
+        msg = f"memmapping is not implemented for {self.__class__.__name__}"
+        raise NotImplementedError(msg)
+
+    def flush_memmap(self):
+        """
+        Flush any pending writes to the memmapped file (if one was mapped with
+        memmap_array)
+        """
+        msg = f"memmapping is not implemented for {self.__class__.__name__}"
+        raise NotImplementedError(msg)
 
     def read_into_array(self, size):
         """
@@ -627,7 +680,7 @@ class GenericFile(metaclass=util.InheritDocstrings):
 
         Returns
         -------
-        array : np.core.memmap
+        array : np.memmap
         """
         buff = self.read(size)
         return np.frombuffer(buff, np.uint8, size, 0)
@@ -645,7 +698,7 @@ class GenericWrapper:
     def __enter__(self):
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, type_, value, traceback):
         pass
 
     def __getattr__(self, attr):
@@ -661,7 +714,13 @@ class RandomAccessFile(GenericFile):
         return True
 
     def reader_until(
-        self, delimiter, readahead_bytes, delimiter_name=None, include=True, initial_content=b"", exception=True
+        self,
+        delimiter,
+        readahead_bytes,
+        delimiter_name=None,
+        include=True,
+        initial_content=b"",
+        exception=True,
     ):
         return _TruncatedReader(
             self,
@@ -678,41 +737,16 @@ class RandomAccessFile(GenericFile):
             self.seek(0, SEEK_END)
         self.seek(size, SEEK_CUR)
 
-    if sys.platform.startswith("win"):  # pragma: no cover
-
-        def truncate(self, size=None):
-            # ftruncate doesn't work on an open file in Windows.  The
-            # best we can do is clear the extra bytes or add extra
-            # bytes to the end.
-            if size is None:
-                size = self.tell()
-
-            self.seek(0, SEEK_END)
-            file_size = self.tell()
-            if size < file_size:
-                self.seek(size, SEEK_SET)
-                nbytes = file_size - size
-            elif size > file_size:
-                nbytes = size - file_size
-            else:
-                nbytes = 0
-
-            block = b"\0" * self.block_size
-            while nbytes > 0:
-                self.write(block[: min(nbytes, self.block_size)])
-                nbytes -= self.block_size
-
+    def truncate(self, size=None):
+        # windows supports truncating as long as the file not opened
+        # more than once. So this must be called after closing all
+        # memmaps
+        if size is None:
+            self._fd.truncate()
+        else:
+            self._fd.truncate(size)
             self.seek(size, SEEK_SET)
 
-    else:
-
-        def truncate(self, size=None):
-            if size is None:
-                self._fd.truncate()
-            else:
-                self._fd.truncate(size)
-                self.seek(size, SEEK_SET)
-
 
 class RealFile(RandomAccessFile):
     """
@@ -720,11 +754,9 @@ class RealFile(RandomAccessFile):
     """
 
     def __init__(self, fd, mode, close=False, uri=None):
-        super(RealFile, self).__init__(fd, mode, close=close, uri=uri)
+        super().__init__(fd, mode, close=close, uri=uri)
 
-        stat = os.fstat(fd.fileno())
-        self._size = stat.st_size
-        if uri is None and isinstance(fd.name, str):
+        if uri is None and hasattr(fd, "name") and isinstance(fd.name, str):
             self._uri = util.filepath_to_url(os.path.abspath(fd.name))
 
     def write_array(self, arr):
@@ -733,7 +765,8 @@ class RealFile(RandomAccessFile):
             self.fast_forward(len(arr.data))
         else:
             if len(arr.shape) != 1 or not arr.flags.contiguous:
-                raise ValueError("Requires 1D contiguous array.")
+                msg = "Requires 1D contiguous array."
+                raise ValueError(msg)
 
             self._fd.write(arr.data)
 
@@ -741,17 +774,66 @@ class RealFile(RandomAccessFile):
         return True
 
     def memmap_array(self, offset, size):
-        if "w" in self._mode:
-            mode = "r+"
-        else:
-            mode = "r"
-        mmap = np.memmap(self._fd, mode=mode, offset=offset, shape=size)
-        mmap.fd = self
-        return mmap
+        if not hasattr(self, "_mmap"):
+            loc = self._fd.tell()
+            acc = mmap.ACCESS_WRITE if "w" in self._mode else mmap.ACCESS_READ
+            self._fd.seek(0, 2)
+            nbytes = self._fd.tell()
+            self._mmap = mmap.mmap(self._fd.fileno(), nbytes, access=acc)
+            # on windows mmap seeks to the start of the file so return the file
+            # pointer to this previous location
+            self._fd.seek(loc, 0)
+        return np.ndarray.__new__(np.memmap, shape=size, offset=offset, dtype="uint8", buffer=self._mmap)
+
+    def close_memmap(self):
+        if hasattr(self, "_mmap") and not self._mmap.closed:
+            self._mmap.close()
+            del self._mmap
+
+    def flush_memmap(self):
+        if hasattr(self, "_mmap"):
+            self._mmap.flush()
 
     def read_into_array(self, size):
         return np.fromfile(self._fd, dtype=np.uint8, count=size)
 
+    def _fix_permissions(self):
+        """
+        atomicfile internally uses tempfile.NamedTemporaryFile
+        which uses tempfile.mkstemp which makes a file that is
+
+        "readable and writable only by the creating user ID."
+
+        this creates files with mode 0o600 regardless of umask
+        Rather than modify atomicfile, this will use
+        the umask to determine the file permissions and modify
+        the resulting file permission bits
+        """
+        if isinstance(self._fd, atomicfile._AtomicWFile):
+            fn = self._fd._filename
+            if not os.path.exists(fn):
+                return
+            # there is no way to read the umask without setting it
+            # so set it to the typical default 0o22
+            umask = os.umask(_FILE_PERMISSIONS_DEFAULT_UMASK)
+            if umask != _FILE_PERMISSIONS_DEFAULT_UMASK:
+                # restore the read value if it differs from the default
+                os.umask(umask)
+            permissions = _FILE_PERMISSIONS_ALL if os.path.isdir(fn) else _FILE_PERMISSIONS_NO_EXECUTE
+            os.chmod(self._fd._filename, permissions & ~umask)
+
+    def __exit__(self, type_, value, traceback):
+        super().__exit__(type_, value, traceback)
+        if self._close:
+            self._fix_permissions()
+
+    def close(self):
+        self.flush_memmap()
+        super().close()
+        self.close_memmap()
+        if self._close:
+            self._fix_permissions()
+
 
 class MemoryIO(RandomAccessFile):
     """
@@ -760,11 +842,7 @@ class MemoryIO(RandomAccessFile):
     """
 
     def __init__(self, fd, mode, uri=None):
-        super(MemoryIO, self).__init__(fd, mode, uri=uri)
-        tell = fd.tell()
-        fd.seek(0, 2)
-        self._size = fd.tell()
-        fd.seek(tell, 0)
+        super().__init__(fd, mode, uri=uri)
 
     def read_into_array(self, size):
         buf = self._fd.getvalue()
@@ -782,7 +860,7 @@ class InputStream(GenericFile):
     """
 
     def __init__(self, fd, mode="r", close=False, uri=None):
-        super(InputStream, self).__init__(fd, mode, close=close, uri=uri)
+        super().__init__(fd, mode, close=close, uri=uri)
         self._fd = fd
         self._buffer = b""
 
@@ -804,24 +882,36 @@ class InputStream(GenericFile):
         len_buffer = len(self._buffer)
         if len_buffer == 0:
             return self._fd.read(size)
-        elif size < 0:
+
+        if size < 0:
             self._buffer += self._fd.read()
             buffer = self._buffer
             self._buffer = b""
-            return buffer
-        elif len_buffer < size:
-            if len_buffer < size:
-                self._buffer += self._fd.read(size - len(self._buffer))
-            buffer = self._buffer
-            self._buffer = b""
-            return buffer
-        else:
-            buffer = self._buffer[:size]
-            self._buffer = self._buffer[size:]
+
             return buffer
 
+        if len_buffer < size:
+            if len_buffer < size:
+                self._buffer += self._fd.read(size - len(self._buffer))
+
+            buffer = self._buffer
+            self._buffer = b""
+
+            return buffer
+
+        buffer = self._buffer[:size]
+        self._buffer = self._buffer[size:]
+
+        return buffer
+
     def reader_until(
-        self, delimiter, readahead_bytes, delimiter_name=None, include=True, initial_content=b"", exception=True
+        self,
+        delimiter,
+        readahead_bytes,
+        delimiter_name=None,
+        include=True,
+        initial_content=b"",
+        exception=True,
     ):
         return _TruncatedReader(
             self,
@@ -835,13 +925,14 @@ class InputStream(GenericFile):
 
     def fast_forward(self, size):
         if size >= 0 and len(self.read(size)) != size:
-            raise IOError("Read past end of file")
+            msg = "Read past end of file"
+            raise OSError(msg)
 
     def read_into_array(self, size):
         try:
             # See if Numpy can handle this as a real file first...
             return np.fromfile(self._fd, np.uint8, size)
-        except (IOError, AttributeError):
+        except (OSError, AttributeError):
             # Else, fall back to reading into memory and then
             # returning the Numpy array.
             data = self.read(size)
@@ -860,7 +951,7 @@ class OutputStream(GenericFile):
     """
 
     def __init__(self, fd, close=False, uri=None):
-        super(OutputStream, self).__init__(fd, "w", close=close, uri=uri)
+        super().__init__(fd, "w", close=close, uri=uri)
         self._fd = fd
 
     def fast_forward(self, size):
@@ -909,11 +1000,13 @@ def _http_to_temp(init, mode, uri=None):
                 fd.write(chunk)
                 chunk = response.read(block_size)
         fd.seek(0)
-        return RealFile(fd, mode, close=True, uri=uri or init)
+
     except Exception:
         fd.close()
         raise
 
+    return RealFile(fd, mode, close=True, uri=uri or init)
+
 
 def get_uri(file_obj):
     """
@@ -987,86 +1080,114 @@ def get_file(init, mode="r", uri=None, close=False):
     ValueError, TypeError, IOError
     """
     if mode not in ("r", "w", "rw"):
-        raise ValueError("mode must be 'r', 'w' or 'rw'")
+        msg = "mode must be 'r', 'w' or 'rw'"
+        raise ValueError(msg)
 
     if init in (sys.__stdout__, sys.__stdin__, sys.__stderr__):
         init = os.fdopen(init.fileno(), init.mode + "b")
 
     if isinstance(init, (GenericFile, GenericWrapper)):
         if mode not in init.mode:
-            raise ValueError("File is opened as '{0}', but '{1}' was requested".format(init.mode, mode))
+            msg = f"File is opened as '{init.mode}', but '{mode}' was requested"
+            raise ValueError(msg)
+
         return GenericWrapper(init)
 
-    elif isinstance(init, (str, pathlib.Path)):
+    if isinstance(init, (str, pathlib.Path)):
         parsed = patched_urllib_parse.urlparse(str(init))
         if parsed.scheme in ["http", "https"]:
             if "w" in mode:
-                raise ValueError("HTTP connections can not be opened for writing")
+                msg = "HTTP connections can not be opened for writing"
+                raise ValueError(msg)
+
             return _http_to_temp(init, mode, uri=uri)
-        elif parsed.scheme in _local_file_schemes:
-            if mode == "rw":
-                realmode = "r+b"
-            else:
-                realmode = mode + "b"
+
+        if parsed.scheme in _local_file_schemes:
+            realmode = "r+b" if mode == "rw" else mode + "b"
+
+            # if paths have an extra leading '/' urlparse will
+            # parse them even though they violate rfc8089. This will
+            # lead to errors or writing files to unexpected locations
+            # on non-windows systems
+            if not sys.platform.startswith("win") and parsed.scheme == "" and parsed.netloc != "":
+                msg = f"Invalid path {init}"
+                raise ValueError(msg)
+
             # Windows paths are not URIs, and so they should not be parsed as
             # such. Otherwise, the drive component of the path can get lost.
             # This is not an ideal solution, but we can't use pathlib here
             # because it doesn't handle URIs properly.
-            if sys.platform.startswith("win") and parsed.scheme in string.ascii_letters:
-                realpath = str(init)
-            else:
-                realpath = url2pathname(parsed.path)
-            if mode == "w":
-                fd = atomicfile.atomic_open(realpath, realmode)
-            else:
-                fd = open(realpath, realmode)
-            fd = fd.__enter__()
+            realpath = (
+                str(init)
+                if sys.platform.startswith("win") and parsed.scheme in string.ascii_letters
+                else url2pathname(parsed.path)
+            )
+            try:
+                fd = atomicfile.atomic_open(realpath, realmode) if mode == "w" else open(realpath, realmode)
+
+                fd = fd.__enter__()
+            except FileNotFoundError as e:
+                # atomic_open will create an Exception with an odd looking path
+                # overwrite the error message to make it more informative
+                e.filename = realpath
+                raise e
+
             return RealFile(fd, mode, close=True, uri=uri)
 
-    elif isinstance(init, io.BytesIO):
+    if isinstance(init, io.BytesIO):
         return MemoryIO(init, mode, uri=uri)
 
-    elif isinstance(init, io.StringIO):
-        raise TypeError("io.StringIO objects are not supported.  Use io.BytesIO instead.")
+    if isinstance(init, io.StringIO):
+        msg = "io.StringIO objects are not supported.  Use io.BytesIO instead."
+        raise TypeError(msg)
 
-    elif isinstance(init, io.IOBase):
+    if isinstance(init, io.IOBase):
         if ("r" in mode and not init.readable()) or ("w" in mode and not init.writable()):
-            raise ValueError("File is opened as '{0}', but '{1}' was requested".format(init.mode, mode))
+            msg = f"File is opened as '{init.mode}', but '{mode}' was requested"
+            raise ValueError(msg)
 
         if init.seekable():
-            if isinstance(init, (io.BufferedReader, io.BufferedWriter, io.BufferedRandom)):
-                init2 = init.raw
-            else:
-                init2 = init
-            if isinstance(init2, io.RawIOBase):
-                result = RealFile(init2, mode, uri=uri, close=close)
-            else:
+            init2 = init.raw if hasattr(init, "raw") else init
+
+            if hasattr(init2, "getvalue"):
                 result = MemoryIO(init2, mode, uri=uri)
+            else:
+                # can we call 'fileno'? if not, we can't memmap so don't
+                # make a RealFile
+                try:
+                    init2.fileno()
+                    result = RealFile(init2, mode, uri=uri, close=close)
+                except io.UnsupportedOperation:
+                    result = RandomAccessFile(init2, mode, uri=uri, close=close)
+
             result._secondary_fd = init
             return result
-        else:
-            if mode == "w":
-                return OutputStream(init, uri=uri, close=close)
-            elif mode == "r":
-                return InputStream(init, mode, uri=uri, close=close)
-            else:
-                raise ValueError("File '{0}' could not be opened in 'rw' mode".format(init))
 
-    elif mode == "w" and (hasattr(init, "write") and hasattr(init, "seek") and hasattr(init, "tell")):
+        if mode == "w":
+            return OutputStream(init, uri=uri, close=close)
+
+        if mode == "r":
+            return InputStream(init, mode, uri=uri, close=close)
+
+        msg = f"File '{init}' could not be opened in 'rw' mode"
+        raise ValueError(msg)
+
+    if mode == "w" and (hasattr(init, "write") and hasattr(init, "seek") and hasattr(init, "tell")):
         return MemoryIO(init, mode, uri=uri)
 
-    elif mode == "r" and (hasattr(init, "read") and hasattr(init, "seek") and hasattr(init, "tell")):
+    if mode == "r" and (hasattr(init, "read") and hasattr(init, "seek") and hasattr(init, "tell")):
         return MemoryIO(init, mode, uri=uri)
 
-    elif mode == "rw" and (
+    if mode == "rw" and (
         hasattr(init, "read") and hasattr(init, "write") and hasattr(init, "seek") and hasattr(init, "tell")
     ):
         return MemoryIO(init, mode, uri=uri)
 
-    elif mode == "w" and hasattr(init, "write"):
+    if mode == "w" and hasattr(init, "write"):
         return OutputStream(init, uri=uri, close=close)
 
-    elif mode == "r" and hasattr(init, "read"):
+    if mode == "r" and hasattr(init, "read"):
         return InputStream(init, mode, uri=uri, close=close)
 
-    raise ValueError("Can't handle '{0}' as a file for mode '{1}'".format(init, mode))
+    msg = f"Can't handle '{init}' as a file for mode '{mode}'"
+    raise ValueError(msg)
