@@ -5,7 +5,7 @@ as schemas.
 import pkgutil
 from collections.abc import Mapping
 
-from asdf_standard import DirectoryResourceMapping
+from asdf_standard import DirectoryResourceMapping as _DirectoryResourceMapping
 
 from .util import get_class_name
 
@@ -14,10 +14,17 @@ __all__ = [
     "DirectoryResourceMapping",
     "ResourceManager",
     "JsonschemaResourceMapping",
-    "get_json_schema_resource_mappings",
 ]
 
 
+class DirectoryResourceMapping(_DirectoryResourceMapping):
+    """
+    A resource mapping that reads resource content from a directory or directory tree.
+
+    See :class:`~asdf_standard.resource.DirectoryResourceMapping` for details.
+    """
+
+
 class ResourceMappingProxy(Mapping):
     """
     Wrapper around a resource mapping that carries
@@ -26,15 +33,16 @@ class ResourceMappingProxy(Mapping):
     """
 
     @classmethod
-    def maybe_wrap(self, delegate):
+    def maybe_wrap(cls, delegate):
         if isinstance(delegate, ResourceMappingProxy):
             return delegate
-        else:
-            return ResourceMappingProxy(delegate)
+
+        return ResourceMappingProxy(delegate)
 
     def __init__(self, delegate, package_name=None, package_version=None):
         if not isinstance(delegate, Mapping):
-            raise TypeError("Resource mapping must implement the Mapping interface")
+            msg = "Resource mapping must implement the Mapping interface"
+            raise TypeError(msg)
 
         self._delegate = delegate
         self._package_name = package_name
@@ -99,23 +107,19 @@ class ResourceMappingProxy(Mapping):
     def __eq__(self, other):
         if isinstance(other, ResourceMappingProxy):
             return other.delegate is self.delegate
-        else:
-            return False
+
+        return False
 
     def __hash__(self):
         return hash(id(self.delegate))
 
     def __repr__(self):
         if self.package_name is not None:
-            package_description = "{}=={}".format(self.package_name, self.package_version)
+            package_description = f"{self.package_name}=={self.package_version}"
         else:
             package_description = "(none)"
 
-        return "<ResourceMappingProxy class: {} package: {} len: {}>".format(
-            self.class_name,
-            package_description,
-            len(self),
-        )
+        return f"<ResourceMappingProxy class: {self.class_name} package: {package_description} len: {len(self)}>"
 
 
 class ResourceManager(Mapping):
@@ -141,7 +145,8 @@ class ResourceManager(Mapping):
 
     def __getitem__(self, uri):
         if uri not in self._mappings_by_uri:
-            raise KeyError("Resource unavailable for URI: {}".format(uri))
+            msg = f"Resource unavailable for URI: {uri}"
+            raise KeyError(msg)
 
         content = self._mappings_by_uri[uri][uri]
         if isinstance(content, str):
@@ -160,7 +165,7 @@ class ResourceManager(Mapping):
         return uri in self._mappings_by_uri
 
     def __repr__(self):
-        return "<ResourceManager len: {}>".format(self.__len__())
+        return f"<ResourceManager len: {self.__len__()}>"
 
 
 _JSONSCHEMA_URI_TO_FILENAME = {
@@ -176,7 +181,7 @@ class JsonschemaResourceMapping(Mapping):
 
     def __getitem__(self, uri):
         filename = _JSONSCHEMA_URI_TO_FILENAME[uri]
-        return pkgutil.get_data("jsonschema", "schemas/{}".format(filename))
+        return pkgutil.get_data("asdf._jsonschema", f"schemas/{filename}")
 
     def __len__(self):
         return len(_JSONSCHEMA_URI_TO_FILENAME)
@@ -186,9 +191,3 @@ class JsonschemaResourceMapping(Mapping):
 
     def __repr__(self):
         return "JsonschemaResourceMapping()"
-
-
-def get_json_schema_resource_mappings():
-    return [
-        JsonschemaResourceMapping(),
-    ]
