@@ -6,52 +6,22 @@ from collections import OrderedDict
 from collections.abc import Mapping
 from functools import lru_cache
 from numbers import Integral
+from operator import methodcaller
 
 import numpy as np
 import yaml
-from jsonschema import validators as mvalidators
-from jsonschema.exceptions import ValidationError
 
-from . import (
-    constants,
-    extension,
-    generic_io,
-    reference,
-    tagged,
-    treeutil,
-    util,
-    versioning,
-    yamlutil,
-)
+from asdf._jsonschema import validators as mvalidators
+from asdf._jsonschema.exceptions import RefResolutionError, ValidationError
+
+from . import constants, generic_io, reference, tagged, treeutil, util, versioning, yamlutil
 from .config import get_config
 from .exceptions import AsdfDeprecationWarning, AsdfWarning
 from .util import patched_urllib_parse
 
 YAML_SCHEMA_METASCHEMA_ID = "http://stsci.edu/schemas/yaml-schema/draft-01"
 
-
-if getattr(yaml, "__with_libyaml__", None):  # pragma: no cover
-    _yaml_base_loader = yaml.CSafeLoader
-else:  # pragma: no cover
-    _yaml_base_loader = yaml.SafeLoader
-
-
-__all__ = ["validate", "fill_defaults", "remove_defaults", "check_schema"]
-
-
-def default_ext_resolver(uri):
-    """
-    Resolver that uses tag/url mappings from all installed extensions
-    """
-    # Deprecating this because it doesn't play nicely with the caching on
-    # load_schema(...).
-    warnings.warn(
-        "The 'default_ext_resolver(...)' function is deprecated. Use "
-        "'asdf.extension.get_default_resolver()(...)' instead.",
-        AsdfDeprecationWarning,
-    )
-    return extension.get_default_resolver()(uri)
-
+__all__ = ["validate", "fill_defaults", "remove_defaults", "check_schema", "load_schema"]
 
 PYTHON_TYPE_TO_YAML_TAG = {
     None: "null",
@@ -79,27 +49,31 @@ def _type_to_tag(type_):
     return None
 
 
+def _tag_to_uri(input_str):
+    if not input_str.startswith(constants.STSCI_SCHEMA_TAG_BASE):
+        return input_str
+    warnings.warn(
+        "Resolving by tag is deprecated. Use uris instead of tags",
+        AsdfDeprecationWarning,
+    )
+    return f"http://stsci.edu/schemas/asdf{input_str[len(constants.STSCI_SCHEMA_TAG_BASE):]}"
+
+
 def validate_tag(validator, tag_pattern, instance, schema):
     """
     Implements the tag validation directive, which checks the
     tag against a pattern that may include wildcards.  See
     `asdf.util.uri_match` for details on the matching behavior.
     """
-    if hasattr(instance, "_tag"):
-        instance_tag = instance._tag
-    else:
-        # Try tags for known Python builtins
-        instance_tag = _type_to_tag(type(instance))
+    instance_tag = instance._tag if hasattr(instance, "_tag") else _type_to_tag(type(instance))
 
     if instance_tag is None:
         yield ValidationError(
-            "mismatched tags, wanted '{}', got unhandled object type '{}'".format(
-                tag_pattern, util.get_class_name(instance)
-            )
+            f"mismatched tags, wanted '{tag_pattern}', got unhandled object type '{util.get_class_name(instance)}'",
         )
 
     if not util.uri_match(tag_pattern, instance_tag):
-        yield ValidationError("mismatched tags, wanted '{0}', got '{1}'".format(tag_pattern, instance_tag))
+        yield ValidationError(f"mismatched tags, wanted '{tag_pattern}', got '{instance_tag}'")
 
 
 def validate_propertyOrder(validator, order, instance, schema):
@@ -156,11 +130,22 @@ def validate_type(validator, types, instance, schema):
     an error, otherwise falling back to the default type checker.
     """
     if isinstance(instance, datetime.datetime) and schema.get("format") == "date-time" and "string" in types:
-        return
+        return None
 
     return mvalidators.Draft4Validator.VALIDATORS["type"](validator, types, instance, schema)
 
 
+def validate_enum(validator, enums, instance, schema):
+    """
+    `asdf.tagged.Tagged` objects will fail in the default enum validator
+    """
+
+    if isinstance(instance, tagged.Tagged):
+        instance = instance.base
+
+    yield from mvalidators.Draft4Validator.VALIDATORS["enum"](validator, enums, instance, schema)
+
+
 YAML_VALIDATORS = util.HashableDict(mvalidators.Draft4Validator.VALIDATORS.copy())
 YAML_VALIDATORS.update(
     {
@@ -169,7 +154,8 @@ YAML_VALIDATORS.update(
         "flowStyle": validate_flowStyle,
         "style": validate_style,
         "type": validate_type,
-    }
+        "enum": validate_enum,
+    },
 )
 
 
@@ -177,12 +163,11 @@ def validate_fill_default(validator, properties, instance, schema):
     if not validator.is_type(instance, "object"):
         return
 
-    for property, subschema in properties.items():
+    for property_, subschema in properties.items():
         if "default" in subschema:
-            instance.setdefault(property, subschema["default"])
+            instance.setdefault(property_, subschema["default"])
 
-    for err in mvalidators.Draft4Validator.VALIDATORS["properties"](validator, properties, instance, schema):
-        yield err
+    yield from mvalidators.Draft4Validator.VALIDATORS["properties"](validator, properties, instance, schema)
 
 
 FILL_DEFAULTS = util.HashableDict()
@@ -195,13 +180,11 @@ def validate_remove_default(validator, properties, instance, schema):
     if not validator.is_type(instance, "object"):
         return
 
-    for property, subschema in properties.items():
-        if subschema.get("default", None) is not None:
-            if instance.get(property, None) == subschema["default"]:
-                del instance[property]
+    for property_, subschema in properties.items():
+        if subschema.get("default", None) is not None and instance.get(property_, None) == subschema["default"]:
+            del instance[property_]
 
-    for err in mvalidators.Draft4Validator.VALIDATORS["properties"](validator, properties, instance, schema):
-        yield err
+    yield from mvalidators.Draft4Validator.VALIDATORS["properties"](validator, properties, instance, schema)
 
 
 REMOVE_DEFAULTS = util.HashableDict()
@@ -220,6 +203,8 @@ class _ValidationContext:
     when exiting the outermost context.
     """
 
+    __slots__ = ["_depth", "_seen"]
+
     def __init__(self):
         self._depth = 0
         self._seen = set()
@@ -252,93 +237,95 @@ class _ValidationContext:
         return (id(instance), id(schema))
 
 
-@lru_cache()
+@lru_cache
 def _create_validator(validators=YAML_VALIDATORS, visit_repeat_nodes=False):
-    meta_schema = _load_schema_cached(YAML_SCHEMA_METASCHEMA_ID, extension.get_default_resolver(), False, False)
+    meta_schema = _load_schema_cached(YAML_SCHEMA_METASCHEMA_ID, _tag_to_uri, False)
 
     type_checker = mvalidators.Draft4Validator.TYPE_CHECKER.redefine_many(
         {
-            "array": lambda checker, instance: isinstance(instance, list) or isinstance(instance, tuple),
+            "array": lambda checker, instance: isinstance(instance, (list, tuple)),
             "integer": lambda checker, instance: not isinstance(instance, bool) and isinstance(instance, Integral),
             "string": lambda checker, instance: isinstance(instance, (str, np.str_)),
-        }
+        },
     )
     id_of = mvalidators.Draft4Validator.ID_OF
-    base_cls = mvalidators.create(
-        meta_schema=meta_schema, validators=validators, type_checker=type_checker, id_of=id_of
+    ASDFvalidator = mvalidators.create(
+        meta_schema=meta_schema,
+        validators=validators,
+        type_checker=type_checker,
+        id_of=id_of,
     )
 
-    class ASDFValidator(base_cls):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, **kwargs)
-            self._context = _ValidationContext()
+    def _patch_init(cls):
+        original_init = cls.__init__
 
-        def iter_errors(self, instance, _schema=None):
+        def init(self, *args, **kwargs):
+            self.ctx = kwargs.pop("ctx", None)
+            self.serialization_context = kwargs.pop("serialization_context", None)
+
+            original_init(self, *args, **kwargs)
+
+        cls.__init__ = init
+
+    def _patch_iter_errors(cls):
+        original_iter_errors = cls.iter_errors
+
+        cls._context = _ValidationContext()
+
+        def iter_errors(self, instance, *args, **kwargs):
             # We can't validate anything that looks like an external reference,
             # since we don't have the actual content, so we just have to defer
             # it for now.  If the user cares about complete validation, they
             # can call `AsdfFile.resolve_references`.
             with self._context:
-                if _schema is None:
-                    schema = self.schema
-                else:
-                    schema = _schema
-
-                if self._context.seen(instance, schema):
+                if self._context.seen(instance, self.schema):
                     # We've already validated this instance against this schema,
                     # no need to do it again.
                     return
 
                 if not visit_repeat_nodes:
-                    self._context.add(instance, schema)
+                    self._context.add(instance, self.schema)
 
                 if (isinstance(instance, dict) and "$ref" in instance) or isinstance(instance, reference.Reference):
                     return
 
-                if _schema is None:
+                if not self.schema:
                     tag = getattr(instance, "_tag", None)
-                    if tag is not None:
-                        if self.serialization_context.extension_manager.handles_tag(tag):
-                            tag_def = self.serialization_context.extension_manager.get_tag_definition(tag)
-                            schema_uris = tag_def.schema_uris
-                        else:
-                            schema_uris = [self.ctx.tag_mapping(tag)]
-                            if schema_uris[0] == tag:
-                                schema_uris = []
+                    if tag is not None and self.serialization_context.extension_manager.handles_tag_definition(tag):
+                        tag_def = self.serialization_context.extension_manager.get_tag_definition(tag)
+                        schema_uris = tag_def.schema_uris
 
                         # Must validate against all schema_uris
                         for schema_uri in schema_uris:
                             try:
-                                s = _load_schema_cached(schema_uri, self.ctx.resolver, False, False)
-                            except FileNotFoundError:
-                                msg = "Unable to locate schema file for '{}': '{}'"
-                                warnings.warn(msg.format(tag, schema_uri), AsdfWarning)
-                                s = {}
-                            if s:
-                                with self.resolver.in_scope(schema_uri):
-                                    for x in super(ASDFValidator, self).iter_errors(instance, s):
-                                        yield x
+                                with self.resolver.resolving(schema_uri) as resolved:
+                                    yield from self.descend(instance, resolved)
+                            except RefResolutionError:
+                                warnings.warn(f"Unable to locate schema file for '{tag}': '{schema_uri}'", AsdfWarning)
 
                     if isinstance(instance, dict):
                         for val in instance.values():
-                            for x in self.iter_errors(val):
-                                yield x
+                            yield from self.iter_errors(val)
 
                     elif isinstance(instance, list):
                         for val in instance:
-                            for x in self.iter_errors(val):
-                                yield x
+                            yield from self.iter_errors(val)
                 else:
-                    for x in super(ASDFValidator, self).iter_errors(instance, _schema=schema):
-                        yield x
+                    yield from original_iter_errors(self, instance)
 
-    return ASDFValidator
+        cls.iter_errors = iter_errors
+
+    _patch_init(ASDFvalidator)
+    _patch_iter_errors(ASDFvalidator)
+
+    return ASDFvalidator
 
 
-@lru_cache()
+@lru_cache
 def _load_schema(url):
     if url.startswith("http://") or url.startswith("https://") or url.startswith("asdf://"):
-        raise FileNotFoundError("Unable to fetch schema from non-file URL: " + url)
+        msg = f"Unable to fetch schema from non-file URL: {url}"
+        raise FileNotFoundError(msg)
 
     with generic_io.get_file(url) as fd:
         if isinstance(url, str) and url.endswith("json"):
@@ -347,11 +334,14 @@ def _load_schema(url):
         else:
             # The following call to yaml.load is safe because we're
             # using a loader that inherits from pyyaml's SafeLoader.
-            result = yaml.load(fd, Loader=yamlutil.AsdfLoader)  # nosec
+            result = yaml.load(fd, Loader=yamlutil.AsdfLoader)  # noqa: S506
     return result, fd.uri
 
 
 def _make_schema_loader(resolver):
+    if resolver is None:
+        resolver = _tag_to_uri
+
     def load_schema(url):
         # Check if this is a URI provided by the new
         # Mapping API:
@@ -369,7 +359,7 @@ def _make_schema_loader(resolver):
             # doesn't mind.
             # The following call to yaml.load is safe because we're
             # using a loader that inherits from pyyaml's SafeLoader.
-            result = yaml.load(content, Loader=yamlutil.AsdfLoader)  # nosec
+            result = yaml.load(content, Loader=yamlutil.AsdfLoader)  # noqa: S506
             return result, url
 
         # If not, this must be a URL (or missing).  Fall back to fetching
@@ -379,7 +369,7 @@ def _make_schema_loader(resolver):
     return load_schema
 
 
-def _make_resolver(url_mapping):
+def _make_jsonschema_refresolver(url_mapping):
     handlers = {}
     schema_loader = _make_schema_loader(url_mapping)
 
@@ -406,15 +396,7 @@ def _make_resolver(url_mapping):
     )
 
 
-@lru_cache()
-def load_custom_schema(url):
-    warnings.warn(
-        "The 'load_custom_schema(...)' function is deprecated. Use" "'load_schema' instead.", AsdfDeprecationWarning
-    )
-    return load_schema(url, resolve_references=True)
-
-
-def load_schema(url, resolver=None, resolve_references=False, resolve_local_refs=False):
+def load_schema(url, resolver=None, resolve_references=False):
     """
     Load a schema from the given URL.
 
@@ -430,27 +412,16 @@ def load_schema(url, resolver=None, resolve_references=False, resolve_local_refs
         mirror on the local filesystem that you wish to use.
 
     resolve_references : bool, optional
-        If `True`, resolve all `$ref` references.
+        If ``True``, resolve all ``$ref`` references.
 
-    resolve_local_refs : bool, optional
-        If `True`, resolve all `$ref` references that refer to other objects
-        within the same schema. This will automatically be handled when passing
-        `resolve_references=True`, but it may be desirable in some cases to
-        control local reference resolution separately.
-        This parameter is deprecated.
     """
-    if resolve_local_refs is True:
-        warnings.warn("The 'resolve_local_refs' parameter is deprecated.", AsdfDeprecationWarning)
-
     if resolver is None:
-        # We can't just set this as the default in load_schema's definition
-        # because invoking get_default_resolver at import time leads to a circular import.
-        resolver = extension.get_default_resolver()
+        resolver = _tag_to_uri
 
     # We want to cache the work that went into constructing the schema, but returning
     # the same object is treacherous, because users who mutate the result will not
     # expect that they're changing the schema everywhere.
-    return copy.deepcopy(_load_schema_cached(url, resolver, resolve_references, resolve_local_refs))
+    return copy.deepcopy(_load_schema_cached(url, resolver, resolve_references))
 
 
 def _safe_resolve(resolver, json_id, uri):
@@ -471,10 +442,7 @@ def _safe_resolve(resolver, json_id, uri):
     # parse correctly.
     parts = uri.split("#")
     base = parts[0]
-    if len(parts) > 1:
-        fragment = parts[1]
-    else:
-        fragment = ""
+    fragment = parts[1] if len(parts) > 1 else ""
 
     # The generic_io.resolve_uri method cannot operate on tag: URIs.
     # New-style extensions don't support $ref with a tag URI target anyway,
@@ -494,12 +462,14 @@ def _safe_resolve(resolver, json_id, uri):
     return base, fragment
 
 
-@lru_cache()
-def _load_schema_cached(url, resolver, resolve_references, resolve_local_refs):
+@lru_cache
+def _load_schema_cached(url, resolver, resolve_references):
+    if resolver is None:
+        resolver = _tag_to_uri
     loader = _make_schema_loader(resolver)
     schema, url = loader(url)
 
-    if resolve_references or resolve_local_refs:
+    if resolve_references:
 
         def resolve_refs(node, json_id):
             if json_id is None:
@@ -511,16 +481,12 @@ def _load_schema_cached(url, resolver, resolve_references, resolve_local_refs):
                 if suburl_base == url or suburl_base == schema.get("id"):
                     # This is a local ref, which we'll resolve in both cases.
                     subschema = schema
-                elif resolve_references:
-                    # Only resolve non-local refs when the flag is set.
-                    subschema = load_schema(suburl_base, resolver, True)
                 else:
-                    # Otherwise return the $ref unmodified.
-                    return node
+                    subschema = load_schema(suburl_base, resolver, True)
 
                 return reference.resolve_fragment(subschema, suburl_fragment)
-            else:
-                return node
+
+            return node
 
         schema = treeutil.walk_and_modify(schema, resolve_refs)
 
@@ -528,7 +494,7 @@ def _load_schema_cached(url, resolver, resolve_references, resolve_local_refs):
 
 
 def get_validator(
-    schema={},
+    schema=None,
     ctx=None,
     validators=None,
     url_mapping=None,
@@ -541,7 +507,7 @@ def get_validator(
     Get a JSON schema validator object for the given schema.
 
     The additional *args and **kwargs are passed along to
-    `jsonschema.validate`.
+    `~jsonschema.protocols.Validator.validate`.
 
     Parameters
     ----------
@@ -556,8 +522,9 @@ def get_validator(
         A dictionary mapping properties to validators to use (instead
         of the built-in ones and ones provided by extension types).
 
-    url_mapping : resolver.Resolver, optional
-        A resolver to convert remote URLs into local ones.
+    url_mapping : callable, optional
+        A callable that takes one string argument and returns a string
+        to convert remote URLs into local ones.
 
     _visit_repeat_nodes : bool, optional
         Force the validator to visit nodes that it has already
@@ -580,9 +547,9 @@ def get_validator(
 
     if validators is None:
         validators = util.HashableDict(YAML_VALIDATORS.copy())
-        validators.update(ctx.extension_list.validators)
+        validators.update(ctx._extension_manager.validator_manager.get_jsonschema_validators())
 
-    kwargs["resolver"] = _make_resolver(url_mapping)
+    kwargs["resolver"] = _make_jsonschema_refresolver(url_mapping)
 
     # We don't just call validators.validate() directly here, because
     # that validates the schema itself, wasting a lot of time (at the
@@ -590,10 +557,7 @@ def get_validator(
     # test suite!!!).  Instead, we assume that the schemas are valid
     # through the running of the unit tests, not at run time.
     cls = _create_validator(validators=validators, visit_repeat_nodes=_visit_repeat_nodes)
-    validator = cls(schema, *args, **kwargs)
-    validator.ctx = ctx
-    validator.serialization_context = _serialization_context
-    return validator
+    return cls({} if schema is None else schema, *args, ctx=ctx, serialization_context=_serialization_context, **kwargs)
 
 
 def _validate_large_literals(instance, reading):
@@ -613,7 +577,8 @@ def _validate_large_literals(instance, reading):
                 AsdfWarning,
             )
         else:
-            raise ValidationError(f"Integer value {value} is too large to safely represent as a " "literal in ASDF")
+            msg = f"Integer value {value} is too large to safely represent as a literal in ASDF"
+            raise ValidationError(msg)
 
     if isinstance(instance, Integral):
         _validate(instance)
@@ -642,17 +607,18 @@ def _validate_mapping_keys(instance, reading):
                     AsdfWarning,
                 )
             else:
-                raise ValidationError(f"Mapping key {key} is not permitted.  Valid types: " "str, int, bool.")
+                msg = f"Mapping key {key} is not permitted.  Valid types: str, int, bool."
+                raise ValidationError(msg)
 
 
-def validate(instance, ctx=None, schema={}, validators=None, reading=False, *args, **kwargs):
+def validate(instance, ctx=None, schema=None, validators=None, reading=False, *args, **kwargs):
     """
     Validate the given instance (which must be a tagged tree) against
     the appropriate schema.  The schema itself is located using the
     tag on the instance.
 
-    The additional *args and **kwargs are passed along to
-    `jsonschema.validate`.
+    The additional ``*args`` and ``**kwargs`` are passed along to
+    `~jsonschema.protocols.Validator.validate`.
 
     Parameters
     ----------
@@ -679,8 +645,8 @@ def validate(instance, ctx=None, schema={}, validators=None, reading=False, *arg
 
         ctx = AsdfFile()
 
-    validator = get_validator(schema, ctx, validators, ctx.resolver, *args, **kwargs)
-    validator.validate(instance, _schema=(schema or None))
+    validator = get_validator({} if schema is None else schema, ctx, validators, None, *args, **kwargs)
+    validator.validate(instance)
 
     additional_validators = [_validate_large_literals]
     if ctx.version >= versioning.RESTRICTED_KEYS_MIN_VERSION:
@@ -734,7 +700,7 @@ def check_schema(schema, validate_default=True):
     Parameters
     ----------
     schema : dict
-        The schema object, as returned by `load_schema`.
+        The schema object, as returned by ``load_schema``.
     validate_default : bool, optional
         Set to `True` to validate the content of the default
         field against the schema.
@@ -752,22 +718,33 @@ def check_schema(schema, validate_default=True):
                 return
 
             if "default" in instance:
-                with instance_validator.resolver.in_scope(instance_scope):
-                    for err in instance_validator.iter_errors(instance["default"], instance):
-                        yield err
+                instance_validator.resolver.push_scope(instance_scope)
+                try:
+                    yield from instance_validator.descend(instance["default"], instance)
+                finally:
+                    instance_validator.resolver.pop_scope()
 
         validators.update({"default": _validate_default})
 
-    meta_schema_id = schema.get("$schema", YAML_SCHEMA_METASCHEMA_ID)
-    meta_schema = _load_schema_cached(meta_schema_id, extension.get_default_resolver(), False, False)
+        def applicable_validators(schema):
+            items = list(schema.items())
+            items.append(("default", ""))
+            return items
 
-    resolver = _make_resolver(extension.get_default_resolver())
+    else:
+        applicable_validators = methodcaller("items")
+
+    meta_schema_id = schema.get("$schema", YAML_SCHEMA_METASCHEMA_ID)
+    meta_schema = _load_schema_cached(meta_schema_id, _tag_to_uri, False)
+
+    resolver = _make_jsonschema_refresolver(_tag_to_uri)
 
     cls = mvalidators.create(
         meta_schema=meta_schema,
         validators=validators,
         type_checker=mvalidators.Draft4Validator.TYPE_CHECKER,
         id_of=mvalidators.Draft4Validator.ID_OF,
+        applicable_validators=applicable_validators,
     )
     validator = cls(meta_schema, resolver=resolver)
-    validator.validate(schema, _schema=meta_schema)
+    validator.validate(schema)
