@@ -2,6 +2,7 @@ import io
 import os
 import pathlib
 import warnings
+from dataclasses import dataclass
 
 import numpy as np
 import pytest
@@ -14,10 +15,12 @@ def pytest_addoption(parser):
     parser.addini("asdf_schema_root", "Root path indicating where schemas are stored")
     parser.addini("asdf_schema_skip_names", "Base names of files to skip in schema tests")
     parser.addini(
-        "asdf_schema_skip_tests", "List of tests to skip, one per line, in format <schema path suffix>::<test name>"
+        "asdf_schema_skip_tests",
+        "List of tests to skip, one per line, in format <schema path suffix>::<test name>",
     )
     parser.addini(
-        "asdf_schema_xfail_tests", "List of tests to xfail, one per line, in format <schema path suffix>::<test name>"
+        "asdf_schema_xfail_tests",
+        "List of tests to xfail, one per line, in format <schema path suffix>::<test name>",
     )
     parser.addini("asdf_schema_skip_examples", "Base names of schemas whose examples should not be tested")
     parser.addini(
@@ -58,11 +61,10 @@ class AsdfSchemaFile(pytest.File):
         validate_default=True,
         ignore_unrecognized_tag=False,
         ignore_version_mismatch=False,
-        skip_tests=[],
-        xfail_tests=[],
+        skip_tests=None,
+        xfail_tests=None,
         **kwargs,
     ):
-
         # Fix for depreciation of fspath in pytest 7+
         from asdf.util import minversion
 
@@ -82,8 +84,8 @@ class AsdfSchemaFile(pytest.File):
         result.validate_default = validate_default
         result.ignore_unrecognized_tag = ignore_unrecognized_tag
         result.ignore_version_mismatch = ignore_version_mismatch
-        result.skip_tests = skip_tests
-        result.xfail_tests = xfail_tests
+        result.skip_tests = [] if skip_tests is None else skip_tests
+        result.xfail_tests = [] if xfail_tests is None else xfail_tests
 
         return result
 
@@ -122,8 +124,7 @@ class AsdfSchemaFile(pytest.File):
 
         for node in treeutil.iter_tree(schema_tree):
             if isinstance(node, dict) and "examples" in node and isinstance(node["examples"], list):
-                for desc, example in node["examples"]:
-                    yield example
+                yield from node["examples"]
 
 
 class AsdfSchemaItem(pytest.Item):
@@ -141,11 +142,11 @@ class AsdfSchemaItem(pytest.Item):
 
     def runtest(self):
         from asdf import schema
-        from asdf.extension import default_extensions
 
         # Make sure that each schema itself is valid.
         schema_tree = schema.load_schema(
-            self.schema_path, resolver=default_extensions.resolver, resolve_references=True
+            self.schema_path,
+            resolve_references=True,
         )
         schema.check_schema(schema_tree, validate_default=self.validate_default)
 
@@ -153,6 +154,45 @@ class AsdfSchemaItem(pytest.Item):
         return self.fspath, 0, ""
 
 
+@dataclass
+class SchemaExample:
+    description: str
+    example: str
+    _version: str = None
+    other: any = None
+
+    @classmethod
+    def from_schema(cls, example: list):
+        if len(example) == 1:
+            _description = ""
+            _example = example[0]
+        elif len(example) == 2:
+            _description = example[0]
+            _example = example[1]
+            _version = None
+            _other = None
+        elif len(example) > 2:
+            _description = example[0]
+            _example = example[2]
+            _version = example[1]
+            _other = example[3:] if len(example) > 3 else None
+        else:
+            msg = "Invalid example"
+            raise RuntimeError(msg)
+
+        return cls(_description, _example, _version, _other)
+
+    @property
+    def version(self):
+        from asdf import versioning
+
+        if self._version is None:
+            return versioning.default_version
+
+        version = self._version.lower().split("asdf-standard-")[1]
+        return versioning.AsdfVersion(version)
+
+
 class AsdfSchemaExampleItem(pytest.Item):
     @classmethod
     def from_parent(
@@ -172,18 +212,19 @@ class AsdfSchemaExampleItem(pytest.Item):
             result = AsdfSchemaExampleItem(name, parent, **kwargs)
 
         result.filename = str(schema_path)
-        result.example = example
+        result.example = SchemaExample.from_schema(example)
         result.ignore_unrecognized_tag = ignore_unrecognized_tag
         result.ignore_version_mismatch = ignore_version_mismatch
         return result
 
     def runtest(self):
-        from asdf import AsdfFile, block, util
-        from asdf.tests import helpers
+        from asdf import AsdfFile, _block, generic_io, util
+        from asdf._tests import _helpers as helpers
+        from asdf.exceptions import AsdfDeprecationWarning
 
         # Make sure that the examples in the schema files (and thus the
         # ASDF standard document) are valid.
-        buff = helpers.yaml_to_asdf("example: " + self.example.strip())
+        buff = helpers.yaml_to_asdf("example: " + self.example.example.strip(), standard_version=self.example.version)
 
         ff = AsdfFile(
             uri=util.filepath_to_url(os.path.abspath(self.filename)),
@@ -198,21 +239,21 @@ class AsdfSchemaExampleItem(pytest.Item):
             util.filepath_to_url(os.path.abspath(os.path.join(os.path.dirname(self.filename), "external.asdf")))
         ] = ff2
 
-        # Add some dummy blocks so that the ndarray examples work
-        for i in range(3):
-            b = block.Block(np.zeros((1024 * 1024 * 8), dtype=np.uint8))
-            b._used = True
-            ff.blocks.add(b)
-        b._array_storage = "streamed"
+        wb = _block.writer.WriteBlock(np.zeros(1024 * 1024 * 8, dtype=np.uint8))
+        with generic_io.get_file(buff, mode="rw") as f:
+            f.seek(0, 2)
+            _block.writer.write_blocks(f, [wb, wb], streamed_block=wb)
+            f.seek(0)
 
         try:
             # Do not tolerate any warnings that occur during schema validation
             with warnings.catch_warnings():
                 warnings.simplefilter("error")
+                warnings.simplefilter("default", category=AsdfDeprecationWarning)
 
                 ff._open_impl(ff, buff, mode="rw")
         except Exception:
-            print("From file:", self.filename)
+            print(f"Example: {self.example.description}\n From file: {self.filename}")
             raise
 
         # Just test we can write it out.  A roundtrip test
@@ -229,16 +270,16 @@ class AsdfSchemaExampleItem(pytest.Item):
 def _parse_test_list(content):
     result = {}
 
-    for line in content.split("\n"):
-        line = line.strip()
-        if len(line) > 0:
-            parts = line.split("::", 1)
+    if isinstance(content, str):
+        content = content.split("\n")
+
+    for line in content:
+        line_ = line.strip()
+        if len(line_) > 0:
+            parts = line_.split("::", 1)
             path_suffix = pathlib.Path(parts[0]).as_posix()
 
-            if len(parts) == 1:
-                name = "*"
-            else:
-                name = parts[-1]
+            name = "*" if len(parts) == 1 else parts[-1]
 
             if path_suffix not in result:
                 result[path_suffix] = []
@@ -250,11 +291,11 @@ def _parse_test_list(content):
 
 def pytest_collect_file(path, parent):
     if not (parent.config.getini("asdf_schema_tests_enabled") or parent.config.getoption("asdf_tests")):
-        return
+        return None
 
     schema_roots = parent.config.getini("asdf_schema_root").split()
     if not schema_roots:
-        return
+        return None
 
     skip_names = parent.config.getini("asdf_schema_skip_names")
     skip_examples = parent.config.getini("asdf_schema_skip_examples")
